/*
 * Created on: Jul 28, 2024
 */

#include "led.h"
#include "ledprog_pep.h"
#include "ledprog_rgb.h"



#define FL3729_SW_COUNT      4  // switches utilized
#define FL3729_CS_COUNT     16  // current sink outputs used by chip
#define FL3729_G_CURRENT    16  // initial global current setting



const uint8_t cs_currents[FL3729_CS_COUNT] = {
        0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0,
        0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80
};

const uint16_t pwm_cie_256in_1024out[] = {
    0,    0,    1,    1,    2,    2,    3,    3,    4,    4,    4,    5,    5,    6,    6,    7,
    7,    8,    8,    8,    9,    9,   10,   10,   11,   11,   12,   12,   13,   13,   14,   15,
   15,   16,   17,   17,   18,   19,   19,   20,   21,   22,   22,   23,   24,   25,   26,   27,
   28,   29,   30,   31,   32,   33,   34,   35,   36,   37,   38,   39,   40,   42,   43,   44,
   45,   47,   48,   50,   51,   52,   54,   55,   57,   58,   60,   61,   63,   65,   66,   68,
   70,   71,   73,   75,   77,   79,   81,   83,   84,   86,   88,   90,   93,   95,   97,   99,
  101,  103,  106,  108,  110,  113,  115,  118,  120,  123,  125,  128,  130,  133,  136,  138,
  141,  144,  147,  149,  152,  155,  158,  161,  164,  167,  171,  174,  177,  180,  183,  187,
  190,  194,  197,  200,  204,  208,  211,  215,  218,  222,  226,  230,  234,  237,  241,  245,
  249,  254,  258,  262,  266,  270,  275,  279,  283,  288,  292,  297,  301,  306,  311,  315,
  320,  325,  330,  335,  340,  345,  350,  355,  360,  365,  370,  376,  381,  386,  392,  397,
  403,  408,  414,  420,  425,  431,  437,  443,  449,  455,  461,  467,  473,  480,  486,  492,
  499,  505,  512,  518,  525,  532,  538,  545,  552,  559,  566,  573,  580,  587,  594,  601,
  609,  616,  624,  631,  639,  646,  654,  662,  669,  677,  685,  693,  701,  709,  717,  726,
  734,  742,  751,  759,  768,  776,  785,  794,  802,  811,  820,  829,  838,  847,  857,  866,
  875,  885,  894,  903,  913,  923,  932,  942,  952,  962,  972,  982,  992, 1002, 1013, 1023,
};



struct LedMatrix led;
uint8_t rgb[3];

static uint8_t led_matrix_updated = 0;



// helper for LED programs
void use_brightest(uint8_t *dest, uint8_t *compare, uint8_t count)
{
    while (count--) {
        if (*dest < *compare) *dest = *compare;
        dest++;
        compare++;
    }
}

static uint16_t rgb_pwm_gamma(uint8_t in)
{
    return pwm_cie_256in_1024out[in];
}

void led_init()
{
    TIM_TimeBaseInitTypeDef timer ={0};
    TIM_OCInitTypeDef pwm = {0};


    // configure matrix
    is31fl3729_init(FL3729_ADDR,
                    FL3729_CONF_SHDN_DIS | FL3729_CONF_OSDE_OFF | FL3729_CONF_MATRIX_4x16,
                    FL3729_G_CURRENT);

    is31fl3729_set_scaling_current_multi(FL3729_ADDR, (uint8_t *)cs_currents, sizeof(cs_currents));


    // configure rear RGBLED
    timer.TIM_Period = (1 << 10) - 1;       // 10-bit
    timer.TIM_Prescaler = 0;
    timer.TIM_ClockDivision = TIM_CKD_DIV1;
    timer.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInit(RGB_TIM, &timer);

    pwm.TIM_OCMode = TIM_OCMode_PWM1;
    pwm.TIM_OutputState = TIM_OutputState_Enable;
    pwm.TIM_Pulse = 0;
    pwm.TIM_OCPolarity = TIM_OCPolarity_Low;
    TIM_OC1Init(RGB_TIM, &pwm);
    TIM_OC2Init(RGB_TIM, &pwm);
    TIM_OC3Init(RGB_TIM, &pwm);

    TIM_CtrlPWMOutputs(RGB_TIM, ENABLE);
    TIM_OC1PreloadConfig(RGB_TIM, TIM_OCPreload_Disable);
    TIM_ARRPreloadConfig(RGB_TIM, ENABLE);
    TIM_Cmd(TIM1, ENABLE);
}

void led_matrix_is_updated()
{
    led_matrix_updated = 1;
}

void led_matrix_send()
{
    // only render when there's something to render
    if (led_matrix_updated) {
        is31fl3729_set_outputs(FL3729_ADDR, 1, (uint8_t *)&led, FL3729_SW_COUNT * FL3729_CS_COUNT);
    }
}

void led_rgb_update()
{
    // this isn't a matrix so we can just update whenever
    TIM_CtrlPWMOutputs(RGB_TIM, DISABLE);
    RGB_TIM->CH1CVR = rgb_pwm_gamma(rgb[2]);
    RGB_TIM->CH2CVR = rgb_pwm_gamma(rgb[1]);
    RGB_TIM->CH3CVR = rgb_pwm_gamma(rgb[0]);
    TIM_CtrlPWMOutputs(RGB_TIM, ENABLE);
}
